@@ -3,7 +3,7 @@
  *  BlueZ - Bluetooth protocol stack for Linux
  *
  *  Copyright (C) 2006-2007  Nokia Corporation
- *  Copyright (C) 2004-2008  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2004-2009  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -43,12 +43,7 @@ typedef enum {
 	NAME_SENT          /* D-Bus signal RemoteNameUpdated sent */
 } name_status_t;
 
-typedef enum {
-	AUTH_TYPE_PINCODE,
-	AUTH_TYPE_PASSKEY,
-	AUTH_TYPE_CONFIRM,
-	AUTH_TYPE_NOTIFY,
-} auth_type_t;
+struct btd_adapter;
 
 struct remote_dev_info {
 	bdaddr_t bdaddr;
@@ -58,31 +53,6 @@ struct remote_dev_info {
 	name_status_t name_status;
 };
 
-struct bonding_request_info {
-	DBusConnection *conn;
-	DBusMessage *msg;
-	struct btd_adapter *adapter;
-	bdaddr_t bdaddr;
-	GIOChannel *io;
-	guint io_id;
-	guint listener_id;
-	int hci_status;
-	int cancel;
-	int auth_active;
-};
-
-struct pending_auth_info {
-	auth_type_t type;
-	bdaddr_t bdaddr;
-	gboolean replied;	/* If we've already replied to the request */
-	struct agent *agent;    /* Agent associated with the request */
-};
-
-struct active_conn_info {
-	bdaddr_t bdaddr;
-	uint16_t handle;
-};
-
 struct hci_dev {
 	int ignore;
 
@@ -114,6 +84,8 @@ struct btd_device *adapter_get_device(DBusConnection *conn,
 
 struct btd_device *adapter_find_device(struct btd_adapter *adapter, const char *dest);
 
+struct btd_device *adapter_find_connection(struct btd_adapter *adapter, uint16_t handle);
+
 void adapter_remove_device(DBusConnection *conn, struct btd_adapter *adapter,
 				struct btd_device *device);
 struct btd_device *adapter_create_device(DBusConnection *conn,
@@ -123,24 +95,12 @@ int pending_remote_name_cancel(struct btd_adapter *adapter);
 
 void remove_pending_device(struct btd_adapter *adapter);
 
-struct pending_auth_info *adapter_find_auth_request(struct btd_adapter *adapter,
-							bdaddr_t *dba);
-void adapter_remove_auth_request(struct btd_adapter *adapter, bdaddr_t *dba);
-struct pending_auth_info *adapter_new_auth_request(struct btd_adapter *adapter,
-							bdaddr_t *dba,
-							auth_type_t type);
-struct btd_adapter *adapter_create(DBusConnection *conn, int id);
+struct btd_adapter *adapter_create(DBusConnection *conn, int id,
+				gboolean devup);
 void adapter_remove(struct btd_adapter *adapter);
 uint16_t adapter_get_dev_id(struct btd_adapter *adapter);
 const gchar *adapter_get_path(struct btd_adapter *adapter);
 void adapter_get_address(struct btd_adapter *adapter, bdaddr_t *bdaddr);
-void adapter_remove(struct btd_adapter *adapter);
-void adapter_set_discov_timeout(struct btd_adapter *adapter, guint interval);
-void adapter_remove_discov_timeout(struct btd_adapter *adapter);
-void adapter_set_scan_mode(struct btd_adapter *adapter, uint8_t scan_mode);
-uint8_t adapter_get_scan_mode(struct btd_adapter *adapter);
-void adapter_set_mode(struct btd_adapter *adapter, uint8_t mode);
-uint8_t adapter_get_mode(struct btd_adapter *adapter);
 void adapter_set_state(struct btd_adapter *adapter, int state);
 int adapter_get_state(struct btd_adapter *adapter);
 struct remote_dev_info *adapter_search_found_devices(struct btd_adapter *adapter,
@@ -152,18 +112,12 @@ int adapter_remove_found_device(struct btd_adapter *adapter, bdaddr_t *bdaddr);
 void adapter_update_oor_devices(struct btd_adapter *adapter);
 void adapter_remove_oor_device(struct btd_adapter *adapter, char *peer_addr);
 void adapter_mode_changed(struct btd_adapter *adapter, uint8_t scan_mode);
+void adapter_name_changed(struct btd_adapter *adapter, const char *name);
 struct agent *adapter_get_agent(struct btd_adapter *adapter);
-void adapter_add_active_conn(struct btd_adapter *adapter, bdaddr_t *bdaddr,
-				uint16_t handle);
-void adapter_remove_active_conn(struct btd_adapter *adapter,
-				struct active_conn_info *dev);
-void adapter_update_devices(struct btd_adapter *adapter);
-struct active_conn_info *adapter_search_active_conn_by_bdaddr(struct btd_adapter *adapter,
-								bdaddr_t *bda);
-struct active_conn_info *adapter_search_active_conn_by_handle(struct btd_adapter *adapter,
-							uint16_t handle);
-void adapter_free_bonding_request(struct btd_adapter *adapter);
-struct bonding_request_info *adapter_get_bonding_info(struct btd_adapter *adapter);
+void adapter_add_connection(struct btd_adapter *adapter,
+				struct btd_device *device, uint16_t handle);
+void adapter_remove_connection(struct btd_adapter *adapter,
+				struct btd_device *device, uint16_t handle);
 gboolean adapter_has_discov_sessions(struct btd_adapter *adapter);
 
 struct btd_adapter_driver {
@@ -185,4 +139,4 @@ const char *adapter_any_get_path(void);
 const char *btd_adapter_any_request_path(void);
 void btd_adapter_any_release_path(void);
 gboolean adapter_is_pairable(struct btd_adapter *adapter);
-gboolean adapter_pairing_initiator(struct btd_adapter *adapter, bdaddr_t *bda);
+gboolean adapter_powering_down(struct btd_adapter *adapter);
