@@ -57,6 +57,7 @@
 #include "agent.h"
 #include "sdp-xml.h"
 #include "storage.h"
+#include "btio.h"
 
 #define DEFAULT_XML_BUF_SIZE	1024
 #define DISCONNECT_TIMER	2
@@ -73,7 +74,6 @@ struct bonding_req {
 	GIOChannel *io;
 	guint io_id;
 	guint listener_id;
-	gboolean auth_required;
 	struct btd_device *device;
 };
 
@@ -87,6 +87,7 @@ struct authentication_req {
 struct btd_device {
 	bdaddr_t	bdaddr;
 	gchar		*path;
+	char		name[248];
 	struct btd_adapter	*adapter;
 	GSList		*uuids;
 	GSList		*drivers;		/* List of driver_data */
@@ -254,8 +255,8 @@ static DBusMessage *get_properties(DBusConnection *conn,
 	adapter_get_address(adapter, &src);
 	ba2str(&src, srcaddr);
 
-	if (read_device_name(srcaddr, dstaddr, name) == 0) {
-		ptr = name;
+	if (device->name) {
+		ptr = device->name;
 		dict_append_entry(&dict, "Name", DBUS_TYPE_STRING, &ptr);
 	}
 
@@ -621,6 +622,8 @@ struct btd_device *device_create(DBusConnection *conn,
 	gchar *address_up;
 	struct btd_device *device;
 	const gchar *adapter_path = adapter_get_path(adapter);
+	bdaddr_t src;
+	char srcaddr[18];
 
 	device = g_try_malloc0(sizeof(struct btd_device));
 	if (device == NULL)
@@ -642,12 +645,43 @@ struct btd_device *device_create(DBusConnection *conn,
 
 	str2ba(address, &device->bdaddr);
 	device->adapter = adapter;
+	adapter_get_address(adapter, &src);
+	ba2str(&src, srcaddr);
+	read_device_name(srcaddr, address, device->name);
 
 	device->auth = 0xff;
 
 	return device;
 }
 
+void device_set_name(struct btd_device *device, const char *name)
+{
+	DBusConnection *conn = get_dbus_connection();
+	char alias[248];
+	char srcaddr[18], dstaddr[18];
+	bdaddr_t src;
+
+	if (strncmp(name, device->name, 248) == 0)
+		return;
+
+	strncpy(device->name, name, 248);
+
+	emit_property_changed(conn, device->path,
+				DEVICE_INTERFACE, "Name",
+				DBUS_TYPE_STRING, &name);
+
+	adapter_get_address(device->adapter, &src);
+	ba2str(&src, srcaddr);
+	ba2str(&device->bdaddr, dstaddr);
+
+	if (read_device_alias(srcaddr, dstaddr, alias, sizeof(alias)) == 0)
+		return;
+
+	emit_property_changed(conn, device->path,
+				DEVICE_INTERFACE, "Alias",
+				DBUS_TYPE_STRING, &name);
+}
+
 static void device_remove_bonding(struct btd_device *device,
 							DBusConnection *conn)
 {
@@ -987,12 +1021,27 @@ static void append_and_grow_string(void *data, const char *str)
 	buff->data_size += len;
 }
 
-static void discover_device_reply(struct browse_req *req, sdp_list_t *recs)
+static void discover_services_reply(struct browse_req *req, int err,
+							sdp_list_t *recs)
 {
 	DBusMessage *reply;
 	DBusMessageIter iter, dict;
 	sdp_list_t *seq;
 
+	if (err) {
+		const char *err_if;
+
+		if (err == -EHOSTDOWN)
+			err_if = ERROR_INTERFACE ".ConnectionAttemptFailed";
+		else
+			err_if = ERROR_INTERFACE ".Failed";
+
+		reply = dbus_message_new_error(req->msg, err_if,
+							strerror(-err));
+		g_dbus_send_message(req->conn, reply);
+		return;
+	}
+
 	reply = dbus_message_new_method_return(req->msg);
 	if (!reply)
 		return;
@@ -1224,7 +1273,7 @@ proceed:
 
 	if (dbus_message_is_method_call(req->msg, DEVICE_INTERFACE,
 					"DiscoverServices")) {
-		discover_device_reply(req, req->records);
+		discover_services_reply(req, err, req->records);
 		goto cleanup;
 	}
 
@@ -1441,7 +1490,13 @@ DBusMessage *new_authentication_return(DBusMessage *msg, uint8_t status)
 		return dbus_message_new_method_return(msg);
 
 	case 0x04: /* page timeout */
+		return dbus_message_new_error(msg,
+				ERROR_INTERFACE ".ConnectionAttemptFailed",
+				"Page Timeout");
 	case 0x08: /* connection timeout */
+		return dbus_message_new_error(msg,
+				ERROR_INTERFACE ".ConnectionAttemptFailed",
+				"Connection Timeout");
 	case 0x10: /* connection accept timeout */
 	case 0x22: /* LMP response timeout */
 	case 0x28: /* instant passed - is this a timeout? */
@@ -1503,7 +1558,7 @@ static void bonding_request_free(struct bonding_req *bonding, gboolean close)
 
 	if (bonding->io) {
 		if (close)
-			g_io_channel_close(bonding->io);
+			g_io_channel_shutdown(bonding->io, TRUE, NULL);
 		g_io_channel_unref(bonding->io);
 	}
 
@@ -1574,69 +1629,51 @@ proceed:
 	return bonding;
 }
 
-static gboolean create_bonding_io_cb(GIOChannel *io, GIOCondition cond,
-					struct btd_device *device)
+static gboolean bonding_io_cb(GIOChannel *io, GIOCondition cond,
+							gpointer user_data)
 {
+	struct btd_device *device = user_data;
+
+	if (!device->bonding)
+		return FALSE;
+
+	error_connection_attempt_failed(device->bonding->conn,
+					device->bonding->msg, ENETDOWN);
+
+	return FALSE;
+}
+
+static void bonding_connect_cb(GIOChannel *io, GError *err, gpointer user_data)
+{
+	struct btd_device *device = user_data;
 	struct hci_request rq;
 	auth_requested_cp cp;
 	evt_cmd_status rp;
-	struct l2cap_conninfo cinfo;
-	socklen_t len;
-	int sk, dd, ret;
+	int dd;
+	uint16_t handle;
 
 	if (!device->bonding) {
-		g_io_channel_close(io);
-		return FALSE;
+		if (!err)
+			g_io_channel_shutdown(io, TRUE, NULL);
+		return;
 	}
 
-	if (cond & G_IO_NVAL) {
-		if (device->bonding) {
-			DBusMessage *reply;
-
-			reply = new_authentication_return(device->bonding->msg,
-							0x09);
-			g_dbus_send_message(device->bonding->conn, reply);
-		}
-
+	if (err) {
+		error("%s", err->message);
+		error_connection_attempt_failed(device->bonding->conn,
+						device->bonding->msg,
+						ENETDOWN);
 		goto cleanup;
 	}
 
-	if (cond & (G_IO_HUP | G_IO_ERR)) {
-		debug("Hangup or error on bonding IO channel");
-
-		if (device->bonding)
-			error_connection_attempt_failed(device->bonding->conn,
-							device->bonding->msg,
-							ENETDOWN);
-
-		goto failed;
-	}
-
-	sk = g_io_channel_unix_get_fd(io);
-
-	len = sizeof(ret);
-	if (getsockopt(sk, SOL_SOCKET, SO_ERROR, &ret, &len) < 0) {
-		error("Can't get socket error: %s (%d)",
-				strerror(errno), errno);
-		error_failed_errno(device->bonding->conn, device->bonding->msg,
-				errno);
-		goto failed;
-	}
-
-	if (ret != 0) {
-		if (device->bonding)
-			error_connection_attempt_failed(device->bonding->conn,
-							device->bonding->msg,
-							ret);
-		goto failed;
-	}
-
-	len = sizeof(cinfo);
-	if (getsockopt(sk, SOL_L2CAP, L2CAP_CONNINFO, &cinfo, &len) < 0) {
-		error("Can't get connection info: %s (%d)",
-				strerror(errno), errno);
-		error_failed_errno(device->bonding->conn, device->bonding->msg,
-				errno);
+	if (!bt_io_get(io, BT_IO_L2RAW, &err,
+			BT_IO_OPT_HANDLE, &handle,
+			BT_IO_OPT_INVALID)) {
+		error("Unable to get connection handle: %s", err->message);
+		error_connection_attempt_failed(device->bonding->conn,
+						device->bonding->msg,
+						ENETDOWN);
+		g_error_free(err);
 		goto failed;
 	}
 
@@ -1650,7 +1687,7 @@ static gboolean create_bonding_io_cb(GIOChannel *io, GIOCondition cond,
 	memset(&rp, 0, sizeof(rp));
 
 	memset(&cp, 0, sizeof(cp));
-	cp.handle = htobs(cinfo.hci_handle);
+	cp.handle = htobs(handle);
 
 	memset(&rq, 0, sizeof(rq));
 	rq.ogf    = OGF_LINK_CTL;
@@ -1682,20 +1719,17 @@ static gboolean create_bonding_io_cb(GIOChannel *io, GIOCondition cond,
 	hci_close_dev(dd);
 
 	device->bonding->io_id = g_io_add_watch(io,
-						G_IO_NVAL | G_IO_HUP | G_IO_ERR,
-						(GIOFunc) create_bonding_io_cb,
-						device);
+					G_IO_NVAL | G_IO_HUP | G_IO_ERR,
+					bonding_io_cb, device);
 
-	return FALSE;
+	return;
 
 failed:
-	g_io_channel_close(io);
+	g_io_channel_shutdown(io, TRUE, NULL);
 
 cleanup:
 	device->bonding->io_id = 0;
 	bonding_request_free(device->bonding, FALSE);
-
-	return FALSE;
 }
 
 static void create_bond_req_exit(DBusConnection *conn, void *user_data)
@@ -1709,74 +1743,11 @@ static void create_bond_req_exit(DBusConnection *conn, void *user_data)
 
 	if (device->bonding) {
 		device->bonding->listener_id = 0;
-		g_io_channel_close(device->bonding->io);
+		g_io_channel_shutdown(device->bonding->io, TRUE, NULL);
 		bonding_request_free(device->bonding, TRUE);
 	}
 }
 
-static int l2raw_connect(const bdaddr_t *src, const bdaddr_t *dst,
-						gboolean *auth_required)
-{
-	struct sockaddr_l2 addr;
-	long arg;
-	int sk, err, opt;
-
-	sk = socket(PF_BLUETOOTH, SOCK_RAW, BTPROTO_L2CAP);
-	if (sk < 0) {
-		error("Can't create socket: %s (%d)", strerror(errno), errno);
-		return sk;
-	}
-
-	memset(&addr, 0, sizeof(addr));
-	addr.l2_family = AF_BLUETOOTH;
-	bacpy(&addr.l2_bdaddr, src);
-
-	if (bind(sk, (struct sockaddr *) &addr, sizeof(addr)) < 0) {
-		error("Can't bind socket: %s (%d)", strerror(errno), errno);
-		goto failed;
-	}
-
-	opt = L2CAP_LM_AUTH | L2CAP_LM_ENCRYPT | L2CAP_LM_SECURE;
-
-	err = setsockopt(sk, SOL_L2CAP, L2CAP_LM, &opt, sizeof(opt));
-	if (err < 0) {
-		error("setsockopt: %s (%d)", strerror(errno), errno);
-		goto failed;
-	}
-
-	arg = fcntl(sk, F_GETFL);
-	if (arg < 0) {
-		error("Can't get file flags: %s (%d)", strerror(errno), errno);
-		goto failed;
-	}
-
-	arg |= O_NONBLOCK;
-	if (fcntl(sk, F_SETFL, arg) < 0) {
-		error("Can't set file flags: %s (%d)", strerror(errno), errno);
-		goto failed;
-	}
-
-	memset(&addr, 0, sizeof(addr));
-	addr.l2_family = AF_BLUETOOTH;
-	bacpy(&addr.l2_bdaddr, dst);
-
-	if (connect(sk, (struct sockaddr *) &addr, sizeof(addr)) < 0) {
-		if (errno == EAGAIN || errno == EINPROGRESS)
-			return sk;
-		error("Can't connect socket: %s (%d)", strerror(errno), errno);
-		goto failed;
-	}
-
-	if (auth_required)
-		*auth_required = TRUE;
-
-	return sk;
-
-failed:
-	close(sk);
-	return -1;
-}
-
 DBusMessage *device_create_bonding(struct btd_device *device,
 					DBusConnection *conn,
 					DBusMessage *msg,
@@ -1788,8 +1759,8 @@ DBusMessage *device_create_bonding(struct btd_device *device,
 	struct btd_adapter *adapter = device->adapter;
 	struct bonding_req *bonding;
 	bdaddr_t src;
-	int sk;
-	gboolean auth_required;
+	GError *err = NULL;
+	GIOChannel *io;
 
 	adapter_get_address(adapter, &src);
 	ba2str(&src, srcaddr);
@@ -1811,26 +1782,30 @@ DBusMessage *device_create_bonding(struct btd_device *device,
 	}
 
 
-	sk = l2raw_connect(&src, &device->bdaddr, &auth_required);
-	if (sk < 0)
-		return g_dbus_create_error(msg,
+	io = bt_io_connect(BT_IO_L2RAW, bonding_connect_cb, device,
+				NULL, &err,
+				BT_IO_OPT_SOURCE_BDADDR, &src,
+				BT_IO_OPT_DEST_BDADDR, &device->bdaddr,
+				BT_IO_OPT_SEC_LEVEL, BT_IO_SEC_HIGH,
+				BT_IO_OPT_INVALID);
+	if (io == NULL) {
+		DBusMessage *reply;
+		reply = g_dbus_create_error(msg,
 				ERROR_INTERFACE ".ConnectionAttemptFailed",
-				"Connection attempt failed");
+				err->message);
+		error("bt_io_connect: %s", err->message);
+		g_error_free(err);
+		return reply;
+	}
 
 	bonding = bonding_request_new(conn, msg, device, agent_path,
 					capability);
 	if (!bonding) {
-		close(sk);
+		g_io_channel_shutdown(io, TRUE, NULL);
 		return NULL;
 	}
 
-	bonding->auth_required = auth_required;
-
-	bonding->io = g_io_channel_unix_new(sk);
-	bonding->io_id = g_io_add_watch(bonding->io,
-					G_IO_OUT | G_IO_NVAL | G_IO_HUP | G_IO_ERR,
-					(GIOFunc) create_bonding_io_cb,
-					device);
+	bonding->io = io;
 
 	bonding->listener_id = g_dbus_add_disconnect_watch(conn,
 						dbus_message_get_sender(msg),
