@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 2001-2002  Nokia Corporation
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2008  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2009  Marcel Holtmann <marcel@holtmann.org>
  *  Copyright (C) 2002-2003  Stephen Crane <steve.crane@rococosoft.com>
  *
  *
@@ -125,6 +125,13 @@ static void update_svclass_list(void)
 		case INTERCOM_SVCLASS_ID:
 		case FAX_SVCLASS_ID:
 		case SAP_SVCLASS_ID:
+		/*
+		 * Setting the telephony bit for the handsfree audio gateway
+		 * role is not required by the HFP specification, but the
+		 * Nokia 616 carkit is just plain broken! It will refuse
+		 * pairing without this bit set.
+		 */
+		case HANDSFREE_AGW_SVCLASS_ID:
 			val |= 0x40;	/* Telephony */
 			break;
 		case AUDIO_SOURCE_SVCLASS_ID:
@@ -380,6 +387,7 @@ void register_device_id(const uint16_t vendor, const uint16_t product,
 int add_record_to_server(const bdaddr_t *src, sdp_record_t *rec)
 {
 	sdp_data_t *data;
+	sdp_list_t *pattern;
 
 	if (rec->handle == 0xffffffff) {
 		rec->handle = sdp_next_handle();
@@ -403,6 +411,16 @@ int add_record_to_server(const bdaddr_t *src, sdp_record_t *rec)
 		sdp_pattern_add_uuid(rec, &uuid);
 	}
 
+	for (pattern = rec->pattern; pattern; pattern = pattern->next) {
+		char uuid[32];
+
+		if (pattern->data == NULL)
+			continue;
+
+		sdp_uuid2strn((uuid_t *) pattern->data, uuid, sizeof(uuid));
+		debug("Record pattern UUID %s", uuid);
+	}
+
 	update_db_timestamp();
 	update_svclass_list();
 
@@ -430,7 +448,7 @@ int remove_record_from_server(uint32_t handle)
 }
 
 // FIXME: refactor for server-side
-static sdp_record_t *extract_pdu_server(bdaddr_t *device, uint8_t *p, int bufsize, uint32_t handleExpected, int *scanned)
+static sdp_record_t *extract_pdu_server(bdaddr_t *device, uint8_t *p, unsigned int bufsize, uint32_t handleExpected, int *scanned)
 {
 	int extractStatus = -1, localExtractedLength = 0;
 	uint8_t dtd;
@@ -629,10 +647,11 @@ int service_update_req(sdp_req_t *req, sdp_buf_t *rsp)
 			SDPDBG("SvcRecOld : %p", orec);
 			SDPDBG("Failure to update, restore old value");
 
-			if (nrec)
-				sdp_record_free(nrec);
 			status = SDP_INVALID_SYNTAX;
 		}
+
+		if (nrec)
+			sdp_record_free(nrec);
 	} else
 		status = SDP_INVALID_RECORD_HANDLE;
 
